package com.gct.dataengine.controller.MachineEngine;

import com.alibaba.excel.EasyExcel;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.gct.common.core.result.Result;
import com.gct.dataengine.entity.MachineEngine.ImportTemplateListener;
import com.gct.dataengine.entity.MachineEngine.ModbusTcpImportTemplate;
import com.gct.dataengine.entity.OssBase.OssTemplateFile;
import com.gct.dataengine.entity.OssBase.vo.AddTemplateFileVo;
import com.gct.dataengine.service.MachineEngine.MachineTemplateService;
import com.gct.dataengine.template.entity.xml.TemplateDefinition;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.Arrays;

/**
 * @author xusirui 2022/6/20
 */
@RestController
@RequestMapping("/machine/template")
public class MachineTemplateController {

    @Autowired
    MachineTemplateService machineTemplateService;

    @GetMapping("/enabled")
    public Result getEnabledTemplate(){
        return Result.ok(machineTemplateService.getEnabledTemplate());
    }

    @GetMapping("/get")
    public Result getTemplateById(@RequestParam("tempId") String tempId){
        return Result.ok(machineTemplateService.getTemplateById(tempId));
    }

    @PostMapping("/update")
    public Result updateTemplateById(@RequestBody JSONObject param){
        String deviceId = param.getString("tempId");
        TemplateDefinition templateDefinition = JSONObject.parseObject(JSONArray.toJSONString(param.get("templateDefinition")),TemplateDefinition.class);
        return Result.ok(machineTemplateService.updateTemplateById(deviceId,templateDefinition)?"success":"failed");
    }

    @PostMapping("/add")
    public Result addTemplateFile(@RequestBody AddTemplateFileVo addTemplateFileVo){
        return Result.ok(machineTemplateService.addTemplateFileByGroup(addTemplateFileVo));
    }

    @PostMapping("/copy")
    public Result copyTemplateFile(@RequestBody AddTemplateFileVo addTemplateFileVo){
        return Result.ok(machineTemplateService.copyTemplateFileByGroup(addTemplateFileVo));
    }

    @PostMapping("/delete")
    public Result deleteTemplateFile(@RequestBody OssTemplateFile ossTemplateFile){
        return Result.ok(machineTemplateService.deleteTemplateFile(ossTemplateFile));
    }

    @PostMapping("/enable")
    public Result enableTemplate(@RequestBody OssTemplateFile ossTemplateFile){
        return Result.ok(machineTemplateService.enableTemplate(ossTemplateFile));
    }

    @PostMapping("/disable")
    public Result disableTemplate(@RequestBody OssTemplateFile ossTemplateFile){
        return Result.ok(machineTemplateService.disableTemplate(ossTemplateFile));
    }

    @GetMapping("/import-temp")
    public void downloadImportTemp(@RequestParam String protocol, HttpServletResponse response) throws IOException {
        response.setContentType("application/vnd.ms-excel");
        response.setCharacterEncoding("utf-8");
        response.setHeader("Content-disposition", "attachment;filename=demo.xlsx");
        EasyExcel.write(response.getOutputStream(), ModbusTcpImportTemplate.class)
                .sheet("sheet1").doWrite(Arrays.asList(new ModbusTcpImportTemplate()));
    }

    @PostMapping("/batch-import")
    public void batchImportTemplateFromExcel(@RequestParam MultipartFile file,@RequestParam String protocol
            ,@RequestParam String baseTemplateId,@RequestParam String groupId) throws IOException {
        EasyExcel.read(file.getInputStream(),ModbusTcpImportTemplate.class,
                new ImportTemplateListener(machineTemplateService,protocol,baseTemplateId,groupId))
                .sheet("sheet1").doRead();
    }
}
