package com.gct.dataengine.service.FileProcess.impl;

import com.alibaba.fastjson.JSONObject;
import com.gct.dataengine.entity.FileProcess.vo.UploadFileVo;
import com.gct.dataengine.entity.OssBase.OssDriverFile;
import com.gct.dataengine.enums.ContentTypeEnum;
import com.gct.dataengine.ossUtil.MinioBucketName;
import com.gct.dataengine.ossUtil.MinioUtil;
import com.gct.dataengine.service.FileProcess.FileProcessService;
import com.gct.dataengine.service.OssBase.OssDriverFileService;
import io.minio.ComposeSource;
import io.minio.messages.DeleteObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.stereotype.Service;

import java.io.File;
import java.io.InputStream;
import java.nio.file.Files;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.UUID;
import java.util.concurrent.RecursiveTask;

/**
 * class name: FileProcessServiceImpl
 *
 * @author gxt
 * @version 1.0
 * @since 2022/5/25 下午3:19 周三
 */
@Service
@EnableConfigurationProperties(MinioBucketName.class)
public class FileProcessServiceImpl implements FileProcessService {

    @Autowired
    MinioUtil minioUtil;

    @Autowired
    MinioBucketName minioBucketName;

    @Autowired
    OssDriverFileService ossDriverFileService;

    @Override
    public String uploadFile(UploadFileVo uploadFileVo) {
        try {
            if (Objects.isNull(uploadFileVo.getFile())) return "上传文件不能为空";
            InputStream inputStream = Files.newInputStream(uploadFileVo.getFile().toPath());
            String bucketName = getBucketName(uploadFileVo.getFile().getName());
            String type = bucketName.substring(bucketName.lastIndexOf(".")+1);
            String res = minioUtil.uploadByInputStream(
                    bucketName,
                    uploadFileVo.getFile().getName(),
                    inputStream,
                    ContentTypeEnum.getContentType(type),
                    uploadFileVo.getCover());
            if ("success".equals(res))
                return minioUtil.previewImgOrDownLoadFile(bucketName, uploadFileVo.getFile().getName());
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    @Override
    public String previewImgOrDownLoadFile(String objectName) {
        String bucketName = getBucketName(objectName);
        return minioUtil.previewImgOrDownLoadFile(bucketName, objectName);
    }

    @Override
    public String partUploadUrl(String objectName, Integer partNum) {
        String bucketName = getBucketName(objectName);
        return minioUtil.getPartUploadUrl(bucketName, objectName + "?partNum=" + partNum);
    }

    @Override
    public boolean composeObject(JSONObject jsonObject) {
        String objectName = jsonObject.getString("objectName");
        String prefix = jsonObject.getString("prefix");
        String bucketName = getBucketName(objectName);
        List<String> listObjects = minioUtil.getListObjects(bucketName, prefix);
        List<ComposeSource> sources = new ArrayList<>();
        List<DeleteObject> deleteObjects = new ArrayList<>();
        for (int i = 0; i < listObjects.size(); i++) {
            sources.add(ComposeSource.builder().bucket(bucketName).object(prefix + i).build());
            deleteObjects.add(new DeleteObject(prefix + i));
        }
        if (minioUtil.composeObjects(bucketName, objectName, sources)) {
            boolean b = minioUtil.deleteObjects(bucketName, deleteObjects);
            if (b) return saveInfoToDriverFile(jsonObject);
        }
        return false;
    }

    private boolean saveInfoToDriverFile(JSONObject jsonObject) {
        String objectName = jsonObject.getString("objectName");
        String bucketName = getBucketName(objectName);
        String url = minioUtil.previewImgOrDownLoadFile(bucketName, objectName);
        OssDriverFile ossDriverFile = new OssDriverFile()
                .setCreator(jsonObject.getString("userName"))
                .setFileName(jsonObject.getString("objectName"))
                .setDriverName(jsonObject.getString("driverName"))
                .setDriverVersion(jsonObject.getDouble("driverVersion"))
                .setCheckType(jsonObject.getString("checkType"))
                .setFilterTag(jsonObject.getString("filterTag"))
                .setUsable(1)
                .setFileLocation(url);
        return ossDriverFileService.saveOrUpdateDriverFile(ossDriverFile);
    }

    public String getBucketName(String objectName) {
        String suffix = objectName.substring(objectName.lastIndexOf(".") + 1);
        String contentType = ContentTypeEnum.getContentType(suffix);
        if (contentType.contains("image"))  return minioBucketName.getBucketImg();
        if (contentType.contains("json"))  return minioBucketName.getBucketJson();
        else return minioBucketName.getBucketFile();
    }

    @Override
    public String uploadImg(File file,String ossName) {
        try {
            assert file != null;
            String bucketName = getBucketName(file.getName());
            int index = file.getName().lastIndexOf(".");
            String substring = file.getName().substring(index + 1);
            String objectName = ossName+"."+substring;
            InputStream inputStream = Files.newInputStream(file.toPath());
            String res = minioUtil.uploadByInputStream(bucketName, objectName, inputStream, ContentTypeEnum.getContentType(substring), "true");
            if ("success".equals(res)) {
                String url = minioUtil.previewImgOrDownLoadFile(bucketName, objectName);
                return url.substring(0,url.lastIndexOf("?"));
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }
}
