package com.gct.dataengine.service.MachineEngine.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.gct.common.util.DateTimeUtil;
import com.gct.common.util.StringUtil;
import com.gct.dataengine.entity.MachineEngine.ModbusTcpImportTemplate;
import com.gct.dataengine.entity.OssBase.GroupTemplateRelation;
import com.gct.dataengine.entity.OssBase.OssTemplateFile;
import com.gct.dataengine.entity.OssBase.vo.AddTemplateFileVo;
import com.gct.dataengine.entity.OssBase.vo.TemplateTaskVo;
import com.gct.dataengine.enums.ContentTypeEnum;
import com.gct.dataengine.ossUtil.MinioBucketName;
import com.gct.dataengine.ossUtil.MinioUtil;
import com.gct.dataengine.service.MachineEngine.MachineTemplateService;
import com.gct.dataengine.service.OssBase.GroupTemplateRelationService;
import com.gct.dataengine.service.OssBase.OssTemplateFileService;
import com.gct.dataengine.template.entity.xml.DataBlockDefinition;
import com.gct.dataengine.template.entity.xml.TemplateDefinition;
import com.gct.dataengine.template.util.DeviceXmlConfigConvert;
import com.gct.dataengine.template.util.XmlConfigParseUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.File;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author xusirui 2022/6/7
 */
@Service
public class MachineMachineTemplateServiceImpl implements MachineTemplateService {

    @Autowired
    MinioUtil minioUtil;

    @Autowired
    MinioBucketName minioBucketName;

    @Autowired
    OssTemplateFileService ossTemplateFileService;

    @Autowired
    GroupTemplateRelationService groupTemplateRelationService;

    private static final String basePath = System.getProperty("user.dir") + "/.temp";

    public MachineMachineTemplateServiceImpl(){
        super();
        File xmlFile = new File(basePath);
        if (!xmlFile.exists() && !xmlFile.mkdirs()) throw new RuntimeException("temp file path create failed");
    }

    @Override
    public TemplateDefinition getTemplateById(String id) {
        OssTemplateFile file = ossTemplateFileService.getById(id);
        if (Objects.isNull(file)) return null;
        String xmlFilePath = minioUtil.getFileToPath(minioBucketName.getBucketFile(), file.getFileName(), basePath);
        DeviceXmlConfigConvert template = new DeviceXmlConfigConvert(xmlFilePath);
        TemplateDefinition templateDefinition = template.read();
        new File(xmlFilePath).delete();
        return templateDefinition;
    }

    @Override
    public boolean updateTemplateById(String id, TemplateDefinition templateDefinition) {
        OssTemplateFile ossTemplateFile = ossTemplateFileService.getById(id);
        if (Objects.isNull(ossTemplateFile)) return false;

        String xmlFilePath = basePath+"/" + ossTemplateFile.getFileName();
        XmlConfigParseUtil.createDefaultXmlFile(xmlFilePath);
        DeviceXmlConfigConvert template = new DeviceXmlConfigConvert(xmlFilePath);
        template.write(templateDefinition);
        String res = minioUtil.uploadByFilePath(minioBucketName.getBucketFile(), ossTemplateFile.getFileName(), xmlFilePath, ContentTypeEnum.DEFAULT.getType(), "true");
        new File(xmlFilePath).delete(); //delete temp file
        ossTemplateFile.setEditStatus(1); // enable make it be 2, edit make it be 1
        ossTemplateFile.setUpdateTime(LocalDateTime.now());
        ossTemplateFile.setUpdater(SecurityContextHolder.getContext().getAuthentication().getName());
        return res.equals("success") && ossTemplateFileService.updateById(ossTemplateFile);
    }

    @Transactional
    @Override
    public boolean deleteTemplateFile(OssTemplateFile ossTemplateFile) {
        return ossTemplateFileService.update(new UpdateWrapper<OssTemplateFile>()
                .eq("id",ossTemplateFile.getId()).set("deleted",1));
    }

    @Override
    public JSONObject enableTemplate(OssTemplateFile ossTemplateFile) {
        TemplateDefinition template = getTemplateById(ossTemplateFile.getId());
        StringBuilder sb = new StringBuilder();
        boolean isValid = Objects.nonNull(template)&&template.validate(sb);
        if (isValid){
            //init to set block id
            template.init();
            System.out.println(template);
            updateTemplateById(ossTemplateFile.getId(),template);
            ossTemplateFile.setEditStatus(2);
            ossTemplateFile.setUpdateTime(LocalDateTime.now());
            ossTemplateFile.setUpdater(SecurityContextHolder.getContext().getAuthentication().getName());
            ossTemplateFileService.updateById(ossTemplateFile);
        }
        JSONObject res = new JSONObject();
        res.put("valid",isValid);
        res.put("message",sb);
        return res;
    }


    @Override
    public boolean disableTemplate(OssTemplateFile ossTemplateFile) {
        ossTemplateFile.setEditStatus(1);
        ossTemplateFile.setUpdateTime(LocalDateTime.now());
        ossTemplateFile.setUpdater(SecurityContextHolder.getContext().getAuthentication().getName());
        return ossTemplateFileService.updateById(ossTemplateFile);
    }

    @Override
    public List<TemplateTaskVo> getEnabledTemplate() {
        return ossTemplateFileService.list(new QueryWrapper<OssTemplateFile>()
                .eq("edit_status", 2)
                .eq("temp_type", "equipmentTemplate")).stream()
                .map(ossTemplateFile -> new TemplateTaskVo(ossTemplateFile.getId(),
                        DateTimeUtil.formatString(ossTemplateFile.getUpdateTime()))).collect(Collectors.toList());
    }

    @Override
    @Transactional
    public boolean addTemplateFileByGroup(AddTemplateFileVo addTemplateFileVo) {
        GroupTemplateRelation groupTemplateRelation = new GroupTemplateRelation();
        OssTemplateFile temp = addTemplateFileVo.getOssTemplateFile();
        String tempId = IdWorker.getIdStr();
        String username = SecurityContextHolder.getContext().getAuthentication().getName();
        groupTemplateRelation.setGroupId(addTemplateFileVo.getGroupId());
        groupTemplateRelation.setTemplateId(tempId);
        groupTemplateRelation.setTempType(temp.getTempType());
        groupTemplateRelation.setCreateTime(LocalDateTime.now());
        groupTemplateRelation.setUpdateTime(LocalDateTime.now());
        groupTemplateRelation.setCreateUser(username);
        String ossFileName = temp.getTempName() + "-" + tempId + ".xml";
        temp.setId(tempId);
        temp.setFileName(ossFileName);
        temp.setFileType("xml");
        temp.setEditStatus(1);
        temp.setDeleted(2);
        temp.setCreator(username);
        temp.setUpdater(username);
        temp.setCreateTime(LocalDateTime.now());
        temp.setUpdateTime(LocalDateTime.now());
        boolean res = ossTemplateFileService.save(temp) && groupTemplateRelationService.save(groupTemplateRelation);
        if (res) res = updateTemplateById(tempId, new TemplateDefinition()); // create a default file
        if (res) {
            String url = minioUtil.previewImgOrDownLoadFile(minioBucketName.getBucketFile(), ossFileName);
            temp.setFileLocation(url);
            ossTemplateFileService.updateById(temp);
        }
        return res;
    }

    @Override
    @Transactional
    public boolean copyTemplateFileByGroup(AddTemplateFileVo addTemplateFileVo) {
        GroupTemplateRelation groupTemplateRelation = new GroupTemplateRelation();
        OssTemplateFile originalTemp = ossTemplateFileService.getById(addTemplateFileVo.getOssTemplateFile().getId());
        String tempId = IdWorker.getIdStr();
        OssTemplateFile copyTemp = new OssTemplateFile();
        String username = SecurityContextHolder.getContext().getAuthentication().getName();
        groupTemplateRelation.setGroupId(addTemplateFileVo.getGroupId());
        groupTemplateRelation.setTemplateId(tempId);
        groupTemplateRelation.setTempType(originalTemp.getTempType());
        groupTemplateRelation.setCreateTime(LocalDateTime.now());
        groupTemplateRelation.setUpdateTime(LocalDateTime.now());
        groupTemplateRelation.setCreateUser(username);
        String ossFileName = addTemplateFileVo.getOssTemplateFile().getTempName() + "-" + tempId + ".xml";
        copyTemp.setId(tempId);
        copyTemp.setFileName(ossFileName);
        copyTemp.setFileType("xml");
        copyTemp.setTempType(originalTemp.getTempType());
        copyTemp.setEditStatus(1);
        copyTemp.setDeleted(2);
        copyTemp.setCreator(username);
        copyTemp.setUpdater(username);
        copyTemp.setCreateTime(LocalDateTime.now());
        copyTemp.setUpdateTime(LocalDateTime.now());
        copyTemp.setTempName(addTemplateFileVo.getOssTemplateFile().getTempName());
        boolean res = ossTemplateFileService.save(copyTemp) && groupTemplateRelationService.save(groupTemplateRelation);
        TemplateDefinition templateDefinition = getTemplateById(originalTemp.getId());
        if (Objects.nonNull(templateDefinition)) {
            if (templateDefinition.getLotServers().length>0 && !StringUtil.isBlankOrEmpty(addTemplateFileVo.getDeviceIpAddress())){
                String url = templateDefinition.getLotServers()[0].getUrl();
                url = url.replace(url.split("://")[1].split("\\?")[0].split(":")[0],addTemplateFileVo.getDeviceIpAddress());
                templateDefinition.getLotServers()[0].setUrl(url);
            }
            for (DataBlockDefinition dataBlockDefinition : templateDefinition.getDataBlockDefinitions()) {
                dataBlockDefinition.setId("#{UUID16}");
            }
        }
        if (res) res = updateTemplateById(tempId, templateDefinition); // create a copy file
        if (res) {
            String url = minioUtil.previewImgOrDownLoadFile(minioBucketName.getBucketFile(), ossFileName);
            copyTemp.setFileLocation(url);
            ossTemplateFileService.updateById(copyTemp);
        }
        return res;
    }

    @Override
    public void copyTemplateFileFromBaseTemp(ModbusTcpImportTemplate modbusTcpImportTemplate, String protocol, String baseTemplateId,String groupId) {
        GroupTemplateRelation groupTemplateRelation = new GroupTemplateRelation();
        OssTemplateFile originalTemp = ossTemplateFileService.getById(baseTemplateId);
        String tempId = IdWorker.getIdStr();
        OssTemplateFile copyTemp = new OssTemplateFile();
        String username = SecurityContextHolder.getContext().getAuthentication().getName();
        groupTemplateRelation.setGroupId(groupId);
        groupTemplateRelation.setTemplateId(tempId);
        groupTemplateRelation.setTempType(originalTemp.getTempType());
        groupTemplateRelation.setCreateTime(LocalDateTime.now());
        groupTemplateRelation.setUpdateTime(LocalDateTime.now());
        groupTemplateRelation.setCreateUser(username);
        String ossFileName = modbusTcpImportTemplate.getName() + "-" + tempId + ".xml";
        copyTemp.setId(tempId);
        copyTemp.setFileName(ossFileName);
        copyTemp.setFileType("xml");
        copyTemp.setTempType(originalTemp.getTempType());
        copyTemp.setEditStatus(1);
        copyTemp.setDeleted(2);
        copyTemp.setCreator(username);
        copyTemp.setUpdater(username);
        copyTemp.setCreateTime(LocalDateTime.now());
        copyTemp.setUpdateTime(LocalDateTime.now());
        copyTemp.setTempName(modbusTcpImportTemplate.getName());
        boolean res = ossTemplateFileService.save(copyTemp) && groupTemplateRelationService.save(groupTemplateRelation);
        TemplateDefinition templateDefinition = getTemplateById(originalTemp.getId());
        if (Objects.nonNull(templateDefinition)) {
            String url = protocol+"://"+modbusTcpImportTemplate.getIp()+":"+modbusTcpImportTemplate.getPort()+"?station="+
                    modbusTcpImportTemplate.getStation()+"&timeout="+modbusTcpImportTemplate.getTimeout()
                    +"&byteOrder="+modbusTcpImportTemplate.getByteOrder();
            for (DataBlockDefinition dataBlockDefinition : templateDefinition.getDataBlockDefinitions()) {
                dataBlockDefinition.setId("#{UUID16}");
            }
            templateDefinition.getLotServers()[0].setUrl(url);
        }
        if (res) res = updateTemplateById(tempId, templateDefinition); // create a copy file
        if (res) {
            String url = minioUtil.previewImgOrDownLoadFile(minioBucketName.getBucketFile(), ossFileName);
            copyTemp.setFileLocation(url);
            ossTemplateFileService.updateById(copyTemp);
        }
    }
}
