package com.gct.dataengine.service.OssBase.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gct.common.util.StringUtil;
import com.gct.dataengine.entity.OssBase.OssDriverFile;
import com.gct.dataengine.entity.OssBase.vo.FileLocationVo;
import com.gct.dataengine.enums.ContentTypeEnum;
import com.gct.dataengine.mapper.OssBase.OssDriverFileMapper;
import com.gct.dataengine.ossUtil.MinioUtil;
import com.gct.dataengine.service.FileProcess.FileProcessService;
import com.gct.dataengine.service.OssBase.OssDriverFileService;
import org.apache.commons.io.IOUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.InputStream;
import java.io.StringWriter;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * <p>
 * 驱动信息存储 服务实现类
 * </p>
 *
 * @author gxt
 * @since 2022-05-30
 */
@Service
public class OssDriverFileServiceImpl extends ServiceImpl<OssDriverFileMapper, OssDriverFile> implements OssDriverFileService {

    @Autowired
    FileProcessService fileProcessService;

    @Autowired
    MinioUtil minioUtil;

    @Override
    public List<OssDriverFile> getList(String driverName, String filterTag,
                                       Integer usable, String fileType) {
        LambdaQueryWrapper<OssDriverFile> wrapper = new LambdaQueryWrapper<>();
        if (!StringUtil.isBlankOrEmpty(driverName)) wrapper = wrapper.eq(OssDriverFile::getDriverName, driverName);
        if (!StringUtil.isBlankOrEmpty(filterTag)) wrapper = wrapper.eq(OssDriverFile::getFilterTag, driverName);
        if (Objects.nonNull(usable)) wrapper = wrapper.eq(OssDriverFile::getUsable, usable);
        if (!StringUtil.isBlankOrEmpty(fileType)) wrapper = wrapper.eq(OssDriverFile::getFileType, fileType);
        wrapper = wrapper.orderByAsc(OssDriverFile::getCreateTime);
        List<OssDriverFile> list = baseMapper.selectList(wrapper);
        return list.isEmpty() ? new ArrayList<>() : list;
    }

    @Override
    public boolean saveOrUpdateDriverFile(OssDriverFile ossDriverFile) {
        String fileLocation = fileProcessService.previewImgOrDownLoadFile(ossDriverFile.getFileName());
        ossDriverFile.setFileLocation(fileLocation);
        if (Objects.isNull(ossDriverFile.getUsable())) ossDriverFile.setUsable(1);
        if (Objects.nonNull(ossDriverFile.getDriverVersion())) {
            ossDriverFile.setDriverVersion(ossDriverFile.getDriverVersion());
        }
        if (!StringUtil.isBlankOrEmpty(ossDriverFile.getFileName())) {
            int index = ossDriverFile.getFileName().lastIndexOf(".");
            ossDriverFile.setFileType(ossDriverFile.getFileName().substring(index + 1));
        }
        return saveOrUpdate(ossDriverFile);
    }

    @Override
    public boolean deleteById(String id) {
        //todo 这里在删除时是否需要把oss上的信息也删除
        return baseMapper.deleteById(id) > 0;
    }

    @Override
    public String getMaxDriverVersion(String driverName) {
        List<OssDriverFile> driverVersion = baseMapper.selectList(new LambdaQueryWrapper<OssDriverFile>()
                .eq(OssDriverFile::getDriverName, driverName)
                .orderByDesc(OssDriverFile::getDriverVersion));
        if (!driverVersion.isEmpty()) {
            return String.valueOf(driverVersion.stream().findFirst().get().getDriverVersion());
        }
        return "";
    }

    @Override
    public FileLocationVo getFileLocation(String driverName, Double limitVersion) {
        OssDriverFile driverFile = this.getOne(new QueryWrapper<OssDriverFile>().eq("driver_name", driverName)
                .ge("driver_version", limitVersion)
                .orderByDesc("driver_version")
                .last("limit 1"));
        if (Objects.isNull(driverFile))return null;
        return new FileLocationVo(fileProcessService.getBucketName(driverFile.getFileName()),driverFile.getFileName());
    }


}
