package com.gct.dataengine.service.OssBase.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gct.dataengine.entity.FileProcess.vo.UploadFileVo;
import com.gct.dataengine.entity.OssBase.GroupTemplateRelation;
import com.gct.dataengine.entity.OssBase.OssTemplateFile;
import com.gct.dataengine.entity.OssBase.vo.AddTemplateFileVo;
import com.gct.dataengine.entity.OssBase.vo.FileLocationVo;
import com.gct.dataengine.entity.OssBase.vo.GetTemplateFileVo;
import com.gct.dataengine.entity.OssBase.vo.TemplateFeignVo;
import com.gct.dataengine.enums.ContentTypeEnum;
import com.gct.dataengine.mapper.OssBase.OssTemplateFileMapper;
import com.gct.dataengine.ossUtil.MinioBucketName;
import com.gct.dataengine.ossUtil.MinioUtil;
import com.gct.dataengine.service.FileProcess.FileProcessService;
import com.gct.dataengine.service.MachineEngine.MachineTemplateService;
import com.gct.dataengine.service.OssBase.GroupTemplateRelationService;
import com.gct.dataengine.service.OssBase.OssTemplateFileService;
import com.gct.dataengine.service.OssBase.OssTemplateGroupService;
import io.minio.messages.DeleteObject;
import org.apache.commons.io.IOUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Service;

import java.io.File;
import java.io.InputStream;
import java.io.StringWriter;
import java.nio.charset.StandardCharsets;
import java.time.LocalDateTime;
import java.util.*;

/**
 * <p>
 * 模板文件存储 服务实现类
 * </p>
 *
 * @author gxt
 * @since 2022-05-30
 */
@Service
public class OssTemplateFileServiceImpl extends ServiceImpl<OssTemplateFileMapper, OssTemplateFile> implements OssTemplateFileService {

    @Autowired
    MinioUtil minioUtil;

    @Autowired
    MinioBucketName minioBucketName;
    @Autowired
    FileProcessService fileProcessService;

    @Autowired
    GroupTemplateRelationService groupTemplateRelationService;

    @Autowired
    MachineTemplateService machineTemplateService;

    @Autowired
    OssTemplateGroupService ossTemplateGroupService;

    @Override
    public Page<Map<String, Object>> getTemplateFileListByGroup(GetTemplateFileVo getTemplateFileVo) {
        return baseMapper.getTemplateFileListByGroup(
                new Page<>(getTemplateFileVo.getCurPage(), getTemplateFileVo.getPageSize())
                , getTemplateFileVo.getTempType(), getTemplateFileVo.getGroupId());
    }

    @Override
    public boolean insertOssTemplateFileInfo(OssTemplateFile ossTemplateFile) {
        UploadFileVo uploadFileVo = new UploadFileVo();
        uploadFileVo.setFile(new File(ossTemplateFile.getTempName()));
        uploadFileVo.setCover("true");
        String result = fileProcessService.uploadFile(uploadFileVo);
        ossTemplateFile.setFileLocation(result);
        return this.saveOrUpdate(ossTemplateFile);
    }

    @Override
    public boolean addMappingTemplate(AddTemplateFileVo addTemplateFileVo) {
        GroupTemplateRelation groupTemplateRelation = new GroupTemplateRelation();
        OssTemplateFile ossTemplateFile = addTemplateFileVo.getOssTemplateFile();
        String tempId = IdWorker.getIdStr();
        String username = SecurityContextHolder.getContext().getAuthentication().getName();
        groupTemplateRelation.setGroupId(addTemplateFileVo.getGroupId());
        groupTemplateRelation.setTemplateId(tempId);
        groupTemplateRelation.setTempType(ossTemplateFile.getTempType());
        groupTemplateRelation.setCreateTime(LocalDateTime.now());
        groupTemplateRelation.setUpdateTime(LocalDateTime.now());
        groupTemplateRelation.setCreateUser(username);
        String ossFileName = ossTemplateFile.getRemark() + "-" + tempId + ".xml";
        ossTemplateFile.setId(tempId);
        ossTemplateFile.setFileName(ossFileName);
        ossTemplateFile.setFileType("xml");
        ossTemplateFile.setEditStatus(1);
        ossTemplateFile.setDeleted(2);
        ossTemplateFile.setCreator(username);
        ossTemplateFile.setUpdater(username);
        ossTemplateFile.setCreateTime(LocalDateTime.now());
        ossTemplateFile.setUpdateTime(LocalDateTime.now());

        return baseMapper.insert(ossTemplateFile) > 0 && groupTemplateRelationService.save(groupTemplateRelation);
    }

    @Override
    public String getFileNameById(String tempId) {
        return baseMapper.selectById(tempId).getFileName().replace(".xml", "");
    }

    @Override
    public boolean deleteOssTemplateFileByNameAndType(String tempName, String tempType) {
        QueryWrapper<OssTemplateFile> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("file_name", tempName).eq("temp_type", tempType);
        String bucketName = fileProcessService.getBucketName(tempName);
        List<DeleteObject> deleteObjects = new ArrayList<>();
        deleteObjects.add(new DeleteObject(tempName));
        List<OssTemplateFile> ossTemplateFiles = baseMapper.selectList(queryWrapper);
        boolean fileDelete = false;
        if (ossTemplateFiles.size() > 0) {
            File file = new File(ossTemplateFiles.get(0).getTempName());
            if (file.exists())
                fileDelete = file.delete();
        }

        return baseMapper.delete(queryWrapper) > 0 && minioUtil.deleteObjects(bucketName, deleteObjects) && fileDelete;
    }

    @Override
    public List<HashMap<String, String>> getTemplateNameByTempType(String tempType) {
        QueryWrapper<OssTemplateFile> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("temp_type", tempType);
        List<OssTemplateFile> ossTemplateFiles = baseMapper.selectList(queryWrapper);
        ArrayList<HashMap<String, String>> list = new ArrayList<>();
        for (OssTemplateFile otf : ossTemplateFiles) {
            HashMap<String, String> hashMap = new HashMap<>();
            hashMap.put("leble", otf.getFileName());
            hashMap.put("value", otf.getFileName());
            hashMap.put("id", otf.getId());
            list.add(hashMap);
        }
        return list;
    }

    @Override
    public boolean isExistByFileName(String fileName, String fieType) {
        QueryWrapper<OssTemplateFile> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("file_name", fileName).eq("temp_type", fieType);
        List<OssTemplateFile> ossTemplateFiles = baseMapper.selectList(queryWrapper);
        return ossTemplateFiles.size() > 0;
    }

    @Override
    public List<OssTemplateFile> getAllTemplateByTempType(String tempType) {
        QueryWrapper<OssTemplateFile> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("temp_type", tempType);
        return baseMapper.selectList(queryWrapper);
    }

    @Override
    public Map<String, String> getJsonDataById(TemplateFeignVo templateFeignVo) {
        Map<String, String> res = new HashMap<>();
        List<OssTemplateFile> list = baseMapper.selectList(new LambdaQueryWrapper<OssTemplateFile>().eq(OssTemplateFile::getId, templateFeignVo.getId()));
        if (!list.isEmpty()) {
            try {
                String fileName = list.stream().findFirst().get().getFileName();
                String bucketName = fileProcessService.getBucketName(fileName);
                InputStream inputStream = minioUtil.getObject(bucketName, fileName);
                StringWriter stringWriter = new StringWriter();
                if (Objects.nonNull(inputStream)) {
                    IOUtils.copy(inputStream, stringWriter, StandardCharsets.UTF_8.name());
                    res.put("json", stringWriter.toString());
                }
                //res.put("tempName", list.get(0).getTempName());
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        res.put("machine", JSON.toJSONString(getMachineInfo(templateFeignVo.getMachineIds())));
        return res;
    }

    private List<Map<String, String>> getMachineInfo(List<String> ids) {
        if (ids.isEmpty()) return new ArrayList<>();
        List<OssTemplateFile> list = baseMapper.selectList(new LambdaQueryWrapper<OssTemplateFile>()
                .in(OssTemplateFile::getId, ids));
        List<Map<String, String>> res = new ArrayList<>();
        if (!list.isEmpty()) {
            list.forEach(x -> {
                Map<String, String> map = new HashMap<>();
                map.put("id", x.getId());
                map.put("tempName", x.getTempName());
                res.add(map);
            });
        }
        return res;
    }

    @Override
    public boolean saveJson(Map<String, String> reqMap) {
        OssTemplateFile ossTemplateFile = new OssTemplateFile();
        String id = reqMap.get("id");
        String json = reqMap.get("json");
        String tempName = reqMap.get("tempName");
        String userName = reqMap.get("userName");
        int index = tempName.lastIndexOf(".");
        String objectName;
        if (index == -1) objectName = tempName + "-" + id + ".json";
        else objectName = tempName.substring(0, index) + "-" + id + tempName.substring(index);
        try {
            InputStream inputStream = IOUtils.toInputStream(json, StandardCharsets.UTF_8.name());
            String bucketName = fileProcessService.getBucketName(objectName);
            String res = minioUtil.uploadByInputStream(bucketName, objectName, inputStream, ContentTypeEnum.getContentType("json"), "true");
            if ("success".equals(res)) {
                String url = minioUtil.getPartUploadUrl(bucketName, objectName);
                ossTemplateFile.setId(id)
                        .setFileName(objectName)
                        .setFileType("json")
                        .setTempType("viewTemplate")
                        .setFileLocation(url)
                        .setDeleted(2)
                        .setEditStatus(1)
                        .setCreator(userName)
                        .setUpdater(userName);
            }
            return saveOrUpdate(ossTemplateFile);
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }
    }

    @Override
    public FileLocationVo getOssTemplateFileById(String tempId) {
        OssTemplateFile file = this.getById(tempId);
        if (Objects.isNull(file)) return new FileLocationVo();
        else return new FileLocationVo(fileProcessService.getBucketName(file.getFileName()), file.getFileName());
    }

    @Override
    public boolean copyObject(Map<String, String> reqMap) {
        String sourceId = reqMap.get("sourceId");
        String targetId = reqMap.get("targetId");
        String sourceName = reqMap.get("sourceName");
        String bucketName = fileProcessService.getBucketName(sourceName);
        String targetName = reqMap.get("targetName");
        boolean b = minioUtil.copyObjects(bucketName, sourceName, bucketName, targetName);
        if (b) {
            String location = minioUtil.previewImgOrDownLoadFile(bucketName, targetName);
            List<OssTemplateFile> sourceList = baseMapper.selectList(new LambdaQueryWrapper<OssTemplateFile>()
                    .eq(OssTemplateFile::getId, sourceId)
                    .eq(OssTemplateFile::getFileName, sourceName));
            if (sourceList.isEmpty()) return false;
            return  baseMapper.insert(sourceList.get(0)
                    .setId(targetId)
                    .setFileName(targetName)
                    .setFileLocation(location)) > 0;
        }
        return false;
    }

    @Override
    public List<Map<String, Object>> getTempByGroupId(String groupId) {
        List<GroupTemplateRelation> grouplist = groupTemplateRelationService
                .list(new QueryWrapper<GroupTemplateRelation>().eq("group_id", groupId));
        List<String> list = new ArrayList<>();
        grouplist.forEach(x -> {
            list.add(x.getTemplateId());
        });
        List<OssTemplateFile> ossTemplateFiles = baseMapper.selectBatchIds(list);
        List<Map<String, Object>> resultList = new ArrayList<>();
        for (OssTemplateFile ossTemplateFile : ossTemplateFiles) {
            Map<String, Object> map = new HashMap<>();
            map.put("id", ossTemplateFile.getId());
            map.put("groupName", ossTemplateFile.getTempName());
            map.put("remark", "template");
            map.put("children", new ArrayList<>());
            resultList.add(map);
        }
        return resultList;
    }

    @Override
    public List<Map<String,Object>> getMatchTempList(String matchStr, String tempType, String groupId) {
        return baseMapper.getMatchTempList(tempType,'%'+matchStr+'%',groupId);
    }

    @Override
    public Boolean batchControlTemplateFileByGroupId(String groupId, Boolean enable, String tempType) {
         baseMapper.batchControlTemplateFileByGroupId(groupId,enable,tempType);
         return true;
    }

    @Override
    public Boolean deleteGroupTempExcludeRoot(String excludeId, String groupId, String tempType) {
        baseMapper.deleteGroupTempExcludeRoot(excludeId,groupId,tempType);
        return true;
    }
}
