package com.gct.dataengine.service.OssBase.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gct.common.util.StringUtil;
import com.gct.dataengine.entity.OssBase.OssTemplateGroup;
import com.gct.dataengine.entity.OssBase.vo.AddTemplateGroupVo;
import com.gct.dataengine.mapper.OssBase.OssTemplateGroupMapper;
import com.gct.dataengine.service.OssBase.GroupTemplateRelationService;
import com.gct.dataengine.service.OssBase.OssTemplateFileService;
import com.gct.dataengine.service.OssBase.OssTemplateGroupService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 模板分组 服务实现类
 * </p>
 *
 * @author gxt
 * @since 2022-05-30
 */
@Service
public class OssTemplateGroupServiceImpl extends ServiceImpl<OssTemplateGroupMapper, OssTemplateGroup> implements OssTemplateGroupService {

    @Autowired
    GroupTemplateRelationService groupTemplateRelationService;
    @Autowired
    OssTemplateFileService ossTemplateFileService;

    @Override
    public List<OssTemplateGroup> getTemplateGroup(String tempType) {
        Map<String,List<OssTemplateGroup>> map = new HashMap<>();
        List<OssTemplateGroup> groups = baseMapper.selectList(new LambdaQueryWrapper<OssTemplateGroup>()
                .eq(OssTemplateGroup::getTempType, tempType));
        groups.forEach(group -> {
            List<OssTemplateGroup> child = map.getOrDefault(group.getParentId(), new ArrayList<>());
            child.add(group);
            map.put(group.getParentId(),child);
        });
        groups.forEach(group -> {
            group.setChildren(map.get(group.getId()));
        });
        return  groups.stream().filter(group -> StringUtil.isBlankOrEmpty(group.getParentId())).collect(Collectors.toList());
    }

    @Override
    public boolean addTemplateGroup(AddTemplateGroupVo addTemplateGroupVo) {
        OssTemplateGroup templateGroup = new OssTemplateGroup();
        templateGroup.setGroupName(addTemplateGroupVo.getGroupName());
        templateGroup.setCreateTime(LocalDateTime.now());
        templateGroup.setCreateTime(LocalDateTime.now());
        templateGroup.setParentId(addTemplateGroupVo.getParentGroupId());
        templateGroup.setRemark(addTemplateGroupVo.getRemark());
        templateGroup.setTempType(addTemplateGroupVo.getTempType());
        templateGroup.setCreator(SecurityContextHolder.getContext().getAuthentication().getName());
        return this.save(templateGroup);
    }

    @Override
    @Transactional
    public boolean deleteTemplateGroup(String groupId) {
        /*LambdaQueryWrapper<GroupTemplateRelation> wrapper = new LambdaQueryWrapper<GroupTemplateRelation>().eq(GroupTemplateRelation::getGroupId, groupId);
        if (! groupTemplateRelationService.getBaseMapper().selectList(wrapper).isEmpty()){
            return groupTemplateRelationService.remove(wrapper) && this.removeById(groupId);
        }
        return this.removeById(groupId);*/
        return baseMapper.deleteById(groupId) > 0;
    }

    @Override
    public List<OssTemplateGroup> getGroupNameList(String tempType) {
        return baseMapper.selectList(new LambdaQueryWrapper<OssTemplateGroup>().eq(OssTemplateGroup::getTempType, tempType));
    }

    @Override
    public List<String> getGroupIdList(String groupId,String tempType) {
        List<OssTemplateGroup> list = getBaseMapper().selectList(
                new LambdaQueryWrapper<OssTemplateGroup>().eq(OssTemplateGroup::getTempType, tempType)
        );
        Set<String> res = new HashSet<>();
        getChildrenGroup(list,groupId,res);
        return new ArrayList<>(res);
    }

    @Override
    public Map<String, String> getAllParentName(List<String> ids, String tempType) {
        List<OssTemplateGroup> groups = baseMapper.selectList(new LambdaQueryWrapper<OssTemplateGroup>()
                .eq(OssTemplateGroup::getTempType, "viewTemplate"));
        Map<String, String> resMap = new HashMap<>();
        ids.forEach(id->{
            StringBuilder stringBuilder = new StringBuilder();
            getAllParentName(groups, id, stringBuilder);
            if (stringBuilder.length() > 0) {
                int index = stringBuilder.toString().lastIndexOf("/");
                resMap.put(id, stringBuilder.substring(0, index));
            }
        });
        return resMap;
    }

    private void getAllParentName(List<OssTemplateGroup> groups, String parId, StringBuilder stringBuilder) {
        Optional<OssTemplateGroup> first = groups.stream().filter(group -> group.getId().equals(parId)).findFirst();
        if (first.isPresent()) {
            OssTemplateGroup temp = first.get();
            stringBuilder.insert(0,temp.getGroupName()+"/");
            getAllParentName(groups, temp.getParentId(), stringBuilder);
        }
    }

    private void getChildrenGroup(List<OssTemplateGroup> groups, String pid, Set<String> res) {
        res.add(pid);
        groups.forEach(group -> {
            if (pid.equals(group.getParentId())) {
                res.add(group.getId());
                getChildrenGroup(groups, group.getId(), res);
            }
        });
    }
}
